BUFX = LibStub("AceAddon-3.0"):NewAddon("BUFX", "AceEvent-3.0","AceHook-3.0")


local AceDB = LibStub("AceDB-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")
local dogTag = LibStub("LibDogTag-3.0")

local options = {
		type='group', childGroups = "tab",
		args={
			square = {
				type = 'toggle', name = "Square",
				desc = "Toggle square frame textures on and off",
				handler = BUFX,
				get = "getSquareState",
				set = "setSquareState",
			},
			xp = {
				type = 'toggle', name = "XP",
				desc = "Toggle XP bar on and off",
				handler = BUFX,
				get = "getXPState",
				set = "setXPState",
			},
			xpcolour = {
				type = 'color', name = "XP Colour",
			    desc = "XP/rep Bar colour",
				handler = BUFX,
			    get = "getXPColour",
			    set = "setXPColour",
			},
			restcolour = {
				type = 'color', name = "Rest Colour",
			    desc = "Rested Bar colour",
				handler = BUFX,
			    get = "getRestColour",
			    set = "setRestColour",
			},
			scale = {
				type = "group", desc = "Options for setting frame scales", name="Scale",
				args = {
					player = {
						type = 'range', name = "Player",
						desc = "Set the scale of the player and target frames",
						handler = BUFX,
						min = 0.6, max = 2,
						--step = 0.1,
						get = "getPlayerScale",
						set = "setPlayerScale",
						--isPercent = true,
					},
					party = {
						type = 'range', name = "Party",
						desc = "Set the scale of the party frames",
						handler = BUFX,
						min = 0.6, max = 2,
						--step = 0.1,
						get = "getPartyScale",
						set = "setPartyScale",
						--isPercent = true,
					},
				},
			},
			smoothbar = {
				type = 'toggle', name = "Smooth Bar",
				desc = "Fades healthbars from green to red as health decreases",
				handler = BUFX,
				get = "getSmoothbarState",
				set = "setSmoothbarState",
			},
			playerPanel = {
				type = 'toggle', name = "Player Panel",
				desc = "Toggle the Player/Pet Panels on and off",
				handler = BUFX,
				get = "getPlayerPanelState",
				set = "setPlayerPanelState",
			},
			targetPanel = {
				type = 'toggle', name = "Target Panel",
				desc = "Toggle the Target Panels on and off",
				handler = BUFX,
				get = "getTargetPanelState",
				set = "setTargetPanelState",
			},
			partyPanel = {
				type = 'toggle', name = "Party Panel",
				desc = "Toggle the Party Panels on and off",
				handler = BUFX,
				get = "getPartyPanelState",
				set = "setPartyPanelState",
			},
		},
	}


function BUFX:OnInitialize()
	self.db = AceDB:New("BUFXDB",{
		profile = {
			square = true,
			xp = nil,
			playerscale = 1,
			partyscale = 1,
			xpcolour = {
				r = 0,
				g = 0.4,
				b = 0.9,
				},
			restcolour = {
				r = 1,
				g = 0.2,
				b = 1,
				},
			smoothbar = nil,
			PlayerPanel = true,
			TargetPanel = true,
			PartyPanel = true,
		}}, "profile")
	AceConfig:RegisterOptionsTable("BUFX", options)
	dialog:AddToBlizOptions("BUFX", "BUFX")
end

	
function BUFX:OnEnable()
	self:doPanels()
	
	PetFrameHappiness:ClearAllPoints()
	PetFrameHappiness:SetPoint("CENTER", "PetFrame", "BOTTOMLEFT")

	if self:getSquareState() then
		self:SetSquare()
	end
	
	if self.db.profile.xp then
		self:XPOn()
	else
		self:XPOff()
	end
	
	TargetFrame:ClearAllPoints()
	TargetFrame:SetPoint("LEFT", "PlayerFrame", "RIGHT", 208, 0)
	
	self:PlayerScale(self:getPlayerScale())
	self:PartyScale(self:getPartyScale())
	
	self:Hook("HealthBar_OnValueChanged", true)
	
	dogTag:AddFontString(BUFX_playerFrameHealthText, BUFX_playerFrame, "[HPColor][FractionalHP]", "Unit", {unit='player'} )
	dogTag:AddFontString(BUFX_playerFrameHealthPercent, BUFX_playerFrame, "[HPColor][PercentHP:Percent]", "Unit", {unit='player'} )
	dogTag:AddFontString(BUFX_playerFrameManaText, BUFX_playerFrame, "[FractionalMP]", "Unit", {unit='player'} )
	
	dogTag:AddFontString(BUFX_targetFrameHealthText, BUFX_targetFrame, "[HPColor][FractionalHP]", "Unit", {unit='target'} )
	dogTag:AddFontString(BUFX_targetFrameHealthPercent, BUFX_targetFrame, "[HPColor][PercentHP:Percent]", "Unit", {unit='target'} )
	dogTag:AddFontString(BUFX_targetFrameManaText, BUFX_targetFrame, "[FractionalMP]", "Unit", {unit='target'} )
	
	dogTag:AddFontString(BUFX_petFrameHealthText, BUFX_petFrame, "[HPColor][FractionalHP]", "Unit", {unit='pet'} )
	
	dogTag:AddFontString(BUFX_party1FrameHealthText, BUFX_party1Frame, "[HPColor][FractionalHP]", "Unit", {unit='party1'} )
	dogTag:AddFontString(BUFX_party2FrameHealthText, BUFX_party2Frame, "[HPColor][FractionalHP]", "Unit", {unit='party2'} )
	dogTag:AddFontString(BUFX_party3FrameHealthText, BUFX_party3Frame, "[HPColor][FractionalHP]", "Unit", {unit='party3'} )
	dogTag:AddFontString(BUFX_party4FrameHealthText, BUFX_party4Frame, "[HPColor][FractionalHP]", "Unit", {unit='party4'} )
end

function BUFX:OnDisable()
	BUFX_playerFrame:Hide()
	BUFX_petFrame:Hide()
	BUFX_targetFrame:Hide()
	BUFX_party1Frame:Hide()
	BUFX_party2Frame:Hide()
	BUFX_party3Frame:Hide()
	BUFX_party4Frame:Hide()
	
	BUFX_XPBarFrame:Hide()
	
	PetFrameHappiness:ClearAllPoints()
	PetFrameHappiness:SetPoint("LEFT", "PetFrame", "RIGHT", -7, -4)
	
	if self:getSquareState() then
		self:SetNormal()
	end
	
	TargetFrame:ClearAllPoints()
	TargetFrame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", 250, -4)
	
	self:PlayerScale(1)
	self:PartyScale(1)
	
	if self:IsHooked("HealthBar_OnValueChanged") then
		self:Unhook("HealthBar_OnValueChanged")
	end
end

function BUFX:OnProfileEnable()
	self:doPanels()
	if self:getSquareState() then
		self:SetSquare()
	else
		self:SetNormal()
	end
	
	if self.db.profile.xp then
		self:XPOn()
	else
		self:XPOff()
	end
	
	self:PlayerScale(self:getPlayerScale())
	self:PartyScale(self:getPartyScale())
end

function BUFX:doPanels()
	if self:getPlayerPanelState() then
		BUFX_playerFrame:Show()
		BUFX_petFrame:Show()
	else
		BUFX_playerFrame:Hide()
		BUFX_petFrame:Hide()
	end
	if self:getTargetPanelState() then
		BUFX_targetFrame:Show()
	else
		BUFX_targetFrame:Hide()
	end
	if self:getPartyPanelState() then
		BUFX_party1Frame:Show()
		BUFX_party2Frame:Show()
		BUFX_party3Frame:Show()
		BUFX_party4Frame:Show()
	else
		BUFX_party1Frame:Hide()
		BUFX_party2Frame:Hide()
		BUFX_party3Frame:Hide()
		BUFX_party4Frame:Hide()
	end
end

function BUFX:TargetFrame_CheckClassification()
	local classification = UnitClassification("target");
	if ( classification == "worldboss" ) then
		TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame-Elite");
	elseif ( classification == "rareelite"  ) then
		TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame-Elite");
	elseif ( classification == "elite"  ) then
		TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame-Elite");
	elseif ( classification == "rare"  ) then
		TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame-Rare");
	else
		TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame");
	end
end

function BUFX:PetFrame_Update()
	if ( UnitManaMax("pet") == 0 ) then
		PetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-SmallTargetingFrame-NoMana");
	else
		PetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-SmallTargetingFrame");
	end
end

function BUFX:HealthBar_OnValueChanged(value, smooth)
	if not smooth then
		smooth = self.db.profile.smoothbar
	end
	self.hooks.HealthBar_OnValueChanged(value, smooth)
end

function BUFX:SetSquare()
	self:SecureHook("TargetFrame_CheckClassification")
	self:SecureHook("PetFrame_Update")

	PlayerFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetingFrame")
	PlayerStatusTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Player-Status")

	PetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-SmallTargetingFrame")
	PetAttackModeTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Player-AttackStatus")

	PartyMemberFrame1Texture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	PartyMemberFrame1Status:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Debuff-Overlays")

	PartyMemberFrame2Texture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	PartyMemberFrame2Status:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Debuff-Overlays")

	PartyMemberFrame3Texture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	PartyMemberFrame3Status:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Debuff-Overlays")

	PartyMemberFrame4Texture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	PartyMemberFrame4Status:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-Debuff-Overlays")

	TargetFrameTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	TargetofTargetTexture:SetTexture("Interface\\AddOns\\BUFX\\texture\\square\\UI-TargetofTargetFrame")
	
	for i=1,4 do
		self:SetPartyMemberPetFrameBack(getglobal("PartyMemberFrame"..i.."PetFrame"),"Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame")
	end 
end

function BUFX:SetNormal()
	self:Unhook("TargetFrame_CheckClassification")
	self:Unhook("PetFrame_Update")

	PlayerFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame")
	PlayerStatusTexture:SetTexture("Interface\\CharacterFrame\\UI-Player-Status")
	PetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-SmallTargetingFrame")
	PetAttackModeTexture:SetTexture("Interface\\TargetingFrame\\UI-Player-AttackStatus")

	PartyMemberFrame1Texture:SetTexture("Interface\\TargetingFrame\\UI-PartyFrame")
	PartyMemberFrame1Status:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")

	PartyMemberFrame2Texture:SetTexture("Interface\\TargetingFrame\\UI-PartyFrame")
	PartyMemberFrame2Status:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")

	PartyMemberFrame3Texture:SetTexture("Interface\\TargetingFrame\\UI-PartyFrame")
	PartyMemberFrame3Status:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")

	PartyMemberFrame4Texture:SetTexture("Interface\\TargetingFrame\\UI-PartyFrame")
	PartyMemberFrame4Status:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")

	TargetFrameTexture:SetTexture("Interface\\TargetingFrame\\UI-PartyFrame")
	TargetofTargetTexture:SetTexture("Interface\\TargetingFrame\\UI-TargetofTargetFrame")
	
	for i=1,4 do
		self:SetPartyMemberPetFrameBack(getglobal("PartyMemberFrame"..i.."PetFrame"),"Interface\\TargetingFrame\\UI-PartyFrame")
	end 
end

function BUFX:XPOn()
	self:RegisterEvent("UPDATE_EXHAUSTION", "XPUpdate")
	self:RegisterEvent("PLAYER_XP_UPDATE", "XPUpdate")
	self:RegisterEvent("UPDATE_FACTION", "XPUpdate")
	
	PetFrame:ClearAllPoints()
	PetFrame:SetPoint("TOPLEFT", "PlayerFrame", "TOPLEFT", 80, -75)
	
	-- if RuneFrame then
	
		-- local point1, frame, point2, x, y = RuneFrame:GetPoint()
		-- if frame:GetName() == "PlayerFrame" then
			-- RuneFrame:ClearAllPoints()
			-- RuneFrame:SetPoint(point1, frame, point2, x, y-12)
		-- end
	-- end
	
	BUFX_XPBarFrame:Show()
	self:ApplyColour()
	self:XPUpdate()
end

function BUFX:XPOff()
	--if self:IsEventRegistered("UPDATE_EXHAUSTION") then
	self:UnregisterEvent("UPDATE_EXHAUSTION")
	self:UnregisterEvent("PLAYER_XP_UPDATE")
	self:UnregisterEvent("UPDATE_FACTION")
	--end
	
	PetFrame:ClearAllPoints()
	PetFrame:SetPoint("TOPLEFT", "PlayerFrame", "TOPLEFT", 80, -60)
	
	BUFX_XPBarFrame:Hide()
end

function BUFX:PlayerScale(s)
	PlayerFrame:SetScale(s)
	TargetFrame:SetScale(s)
end

function BUFX:PartyScale(s)
	PartyMemberFrame1:SetScale(s)
	PartyMemberFrame2:SetScale(s)
	PartyMemberFrame3:SetScale(s)
	PartyMemberFrame4:SetScale(s)
end

function BUFX:SetPartyMemberPetFrameBack(frame,texture) --Provided by Gello, who rocks most hard
	local subFrames = { frame:GetChildren() };
	local subSubFrames, regions

	for _,child in ipairs(subFrames) do
		if child:GetName()==nil then
			subSubFrames = { child:GetChildren() } -- nil child has only one sub-frame, which is anonymous
			regions = { subSubFrames[1]:GetRegions() }

			for _,element in ipairs(regions) do
				if element.GetTexture then
					if element:GetTexture()=="Interface\\TargetingFrame\\UI-PartyFrame" or element:GetTexture()=="Interface\\AddOns\\BUFX\\texture\\square\\UI-PartyFrame" then
						element:SetTexture(texture)
					end
				end
			end

		end
	end
end 

function BUFX:ApplyColour()
	BUFX_ExpBar:SetStatusBarColor(self.db.profile.xpcolour.r, self.db.profile.xpcolour.g, self.db.profile.xpcolour.b)
	BUFX_RestBar:SetStatusBarColor(self.db.profile.restcolour.r, self.db.profile.restcolour.g, self.db.profile.restcolour.b)
end

function BUFX:XPUpdate()
	local name, standing, minRep, maxRep, currentRep = GetWatchedFactionInfo()
	
	if name then
		BUFX_RestBar:Hide()
		BUFX_ExpBar:SetMinMaxValues(0, maxRep - minRep)
		BUFX_ExpBar:SetValue(currentRep - minRep)
		BUFX_XPBarFrame_Exp:SetText(format("%s - %d/%d", name, currentRep - minRep, maxRep - minRep))
	else
		local currentXP = UnitXP("player")
		local maxXP = UnitXPMax("player")
		local restXP = GetXPExhaustion() or 0
		local remainRest = currentXP + restXP
		if remainRest > maxXP then
			remainRest = maxXP
		end
		
		BUFX_ExpBar:SetMinMaxValues(0, maxXP)
		BUFX_ExpBar:SetValue(currentXP)
		if restXP == 0 then
			BUFX_RestBar:Hide()
			BUFX_XPBarFrame_Exp:SetText(format("%d/%d", currentXP, maxXP))
		else
			BUFX_RestBar:Show()
			BUFX_RestBar:SetMinMaxValues(0, maxXP)
			BUFX_RestBar:SetValue(remainRest)
			BUFX_XPBarFrame_Exp:SetText(format("%d/%d (+%d)", currentXP, maxXP, restXP/2))
		end
	end
end

function BUFX:UNIT_PET(target)
	target = target or arg1
	if target == "player" then
		self:UNIT_HEALTH("pet")
	end
end

function BUFX:UNIT_HEALTH(target)
	target = target or arg1

	local healthperc = getglobal(format("BUFX_%sFrameHealthPercent", target))
	local health = getglobal(format("BUFX_%sFrameHealthText", target))
	if self:getFlipState() then
		healthperc, health = health, healthperc
	end
	local ch, mh = UnitHealth(target), UnitHealthMax(target)
	local ph = ch/mh
	
	if target == "target" then
		if MobHealth3 then
			ch, mh = MobHealth3:GetUnitHealth("target", ch, mh)
		end
	end
	if self:getDeficitState() then
		ch = mh - ch
	end
	local r, g
	if(ph > 0.5) then
		r = (1.0 - ph) * 2;
		g = 1.0;
	else
		r = 1.0;
		g = ph * 2;
	end
	if healthperc then
		healthperc:SetText(floor(ph*100).."%")
		healthperc:SetTextColor(r,g,0)
	end
	if health then
		health:SetText(format("%d/%d", ch, mh))
		health:SetTextColor(r,g,0)
	end
end

function BUFX:UNIT_MANA(target)
	target = target or arg1
	local mana = getglobal(format("BUFX_%sFrameManaText", target))
	if mana then
		local cm, mm = UnitMana(target), UnitManaMax(target)
		if self:getDeficitState() then
			cm = mm - cm
		end
	
		mana:SetText(format("%d/%d", cm, mm))
	end
end

function BUFX:PLAYER_TARGET_CHANGED()
	self:UNIT_HEALTH("target")
	self:UNIT_MANA("target")
end

function BUFX:PLAYER_PET_CHANGED()
	if UnitExists("pet") then
	self:UNIT_HEALTH("pet")
	end
end

function BUFX:PARTY_MEMBERS_CHANGED()
	self:UNIT_HEALTH("party1")
	self:UNIT_HEALTH("party2")
	self:UNIT_HEALTH("party3")
	self:UNIT_HEALTH("party4")
end

function BUFX:getSquareState()
	return self.db.profile.square
end

function BUFX:setSquareState()
	self.db.profile.square = not self.db.profile.square
	if self.db.profile.square then
		self:SetSquare()
	else
		self:SetNormal()
	end
end

function BUFX:getXPState()
	return self.db.profile.xp
end

function BUFX:setXPState()
	self.db.profile.xp = not self.db.profile.xp
	if self.db.profile.xp then
		self:XPOn()
	else
		self:XPOff()
	end
end

function BUFX:getXPColour()
	return self.db.profile.xpcolour.r, self.db.profile.xpcolour.g, self.db.profile.xpcolour.b
end

function BUFX:setXPColour(r,g,b)
	self.db.profile.xpcolour.r = r
	self.db.profile.xpcolour.g = g
	self.db.profile.xpcolour.b = b
	self:ApplyColour()
end

function BUFX:getRestColour()
	return self.db.profile.restcolour.r, self.db.profile.restcolour.g, self.db.profile.restcolour.b
end

function BUFX:setRestColour(r,g,b)
	self.db.profile.restcolour.r = r
	self.db.profile.restcolour.g = g
	self.db.profile.restcolour.b = b
	self:ApplyColour()
end

function BUFX:getPlayerScale()
	return self.db.profile.playerscale
end

function BUFX:setPlayerScale(info, s)
	self.db.profile.playerscale = s
	self:PlayerScale(s)
end

function BUFX:getPartyScale()
	return self.db.profile.partyscale
end

function BUFX:setPartyScale(info, s)
	self.db.profile.partyscale = s
	self:PartyScale(s)
end

function BUFX:getFlipState()
	return self.db.profile.flip
end

function BUFX:setFlipState()
	self.db.profile.flip = not self.db.profile.flip
	self:UNIT_HEALTH("player")
	if UnitExists("target") then
		self:UNIT_HEALTH("target")
	end
end

function BUFX:getDeficitState()
	return self.db.profile.deficit
end

function BUFX:setDeficitState()
	self.db.profile.deficit = not self.db.profile.deficit
	self:UNIT_HEALTH("player")
	self:UNIT_MANA("player")
	if UnitExists("target") then
		self:UNIT_HEALTH("target")
		self:UNIT_MANA("target")
	end
	if UnitExists("pet") then
		self:UNIT_HEALTH("pet")
		self:UNIT_MANA("pet")
	end
	if UnitExists("party1") then
		self:UNIT_HEALTH("party1")
		self:UNIT_MANA("party1")
	end
	if UnitExists("party2") then
		self:UNIT_HEALTH("party2")
		self:UNIT_MANA("party1")
	end
	if UnitExists("party3") then
		self:UNIT_HEALTH("party3")
		self:UNIT_MANA("party1")
	end
	if UnitExists("party4") then
		self:UNIT_HEALTH("party4")
		self:UNIT_MANA("party1")
	end
end

function BUFX:getSmoothbarState()
	return self.db.profile.smoothbar
end

function BUFX:setSmoothbarState()
	self.db.profile.smoothbar = not self.db.profile.smoothbar
end

function BUFX:getPlayerPanelState()
	return self.db.profile.playerPanel
end

function BUFX:setPlayerPanelState()
	self.db.profile.playerPanel = not self.db.profile.playerPanel
	self:doPanels()
end

function BUFX:getTargetPanelState()
	return self.db.profile.targetPanel
end

function BUFX:setTargetPanelState()
	self.db.profile.targetPanel = not self.db.profile.targetPanel
	self:doPanels()
end

function BUFX:getPartyPanelState()
	return self.db.profile.partyPanel
end

function BUFX:setPartyPanelState()
	self.db.profile.partyPanel = not self.db.profile.partyPanel
	self:doPanels()
end